// XpaDemoDlg.cpp : main dialog implementation file
//

#include "stdafx.h"
#include "XpaDemo.h"
#include "XpaDemoDlg.h"

#include "XpipeAPI.h"				// XpipeAPI stuff ...

#include "XpaDemoPlaceCallDlg.h"	// Place Call dialog ...
#include "XpaDemoInCallDlg.h"		// Incoming Call dialog ...
#include "XpaDemoClearCallDlg.h"	// Clear Call dialog ...
#include "XpaDemoClearReqDlg.h"		// Clear Request dialog ...
#include "XpaDemoDataDlg.h"			// Data Transfer dialog ...

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// In order to avoid situation where the Clear Request dialog box will be
// displayed more than once i'll keep a global array of BOOL values indexed
// by the logical channel number saying whether the main dialog should display
// the Clear Request dialog box or not. For example if we have Place Call
// dialog open, let's assume than after we've placed the call the peer has
// cleared it ( i.e. rejected the call ) - Place Call dialog will display 
// a message indicating that the call was rejected - we do not want the main
// dialog to display this message again, right ? Now if the main dialog is 
// the only one that is open and we receive Clear Request - in this case
// we do want to display a message box from main dialog. 
//
BOOL	gDisplayClearReq[XPA_MAX_LCN+1] = { TRUE };

/////////////////////////////////////////////////////////////////////////////
// CXpaDemoDlg dialog

CXpaDemoDlg::CXpaDemoDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CXpaDemoDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CXpaDemoDlg)
	m_maxFsize = _T("");
	m_pvcRange = _T("");
	m_station = _T("");
	m_svcRange = _T("");
	m_dataLcn = 1;
	//}}AFX_DATA_INIT
}

void CXpaDemoDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CXpaDemoDlg)
	DDX_Text(pDX, IDC_MAXFSIZE, m_maxFsize);
	DDX_Text(pDX, IDC_PVCRANGE, m_pvcRange);
	DDX_Text(pDX, IDC_STATION, m_station);
	DDX_Text(pDX, IDC_SVCRANGE, m_svcRange);
	DDX_Text(pDX, IDC_DATALCN, m_dataLcn);
	DDV_MinMaxInt(pDX, m_dataLcn, 1, 4095);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CXpaDemoDlg, CDialog)
	//{{AFX_MSG_MAP(CXpaDemoDlg)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_PLACECALL, OnPlaceCallClicked)
	ON_BN_CLICKED(IDC_CLEARCALL, OnClearCallClicked)
	ON_BN_CLICKED(IDC_OPENDATA, OnOpenDataClicked)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WMXPA_ASY_INCALL, OnInCall)			// Incoming call 
	ON_MESSAGE(WMXPA_ASY_CLEARREQ, OnClearReq)		// Clear Request/Indication
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CXpaDemoDlg message handlers

//
// WM_INITDIALOG handler
//
BOOL CXpaDemoDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// register this window with XpipeAPI to receive notifications for
	// all logical channels ...
	if ( !xpaRegisterWindow( m_hWnd, 1, 0 ) ) {
		// failure ... 
		char	szMsgString[MAX_PATH];
		sprintf( szMsgString, "Failed to register the main dialog window with XpipeAPI, last error %d", xpaGetLastError());
		MessageBox( szMsgString );
		//  we have to terminate now 
		EndDialog(0);
	}

	// get configuration data 
	XPA_X25CFG	X25cfg;
	int			iRVal;

	if (0 != (iRVal=xpaReadConfiguration( 1, &X25cfg ))) {
		// failure ...
		char	szMsgString[MAX_PATH];
		sprintf( szMsgString, "Failed to read X.25 configuration. Return value 0x%02X, last error %d", iRVal, xpaGetLastError());
		MessageBox( szMsgString );
	} else {
		// success - set up dlg. data
		m_maxFsize.Format("%d", X25cfg.MaxIFrameSize);
		m_pvcRange.Format("%d to %d", X25cfg.PVClow, X25cfg.PVChigh);
		m_svcRange.Format("%d to %d", X25cfg.SVClow, X25cfg.SVChigh);
		m_station = (X25cfg.DTE) ? "DTE" : "DCE";
		UpdateData( FALSE );
	}
	return TRUE;  // return TRUE  unless you set the focus to a control
}

//
// WM_CLOSE handler
//
void CXpaDemoDlg::OnClose() 
{
	// deregister this window from XpipeAPI
	xpaDeregisterWindow( m_hWnd, 1 );

	CDialog::OnClose();
}

//
// "Place Call" button clicked
//
void CXpaDemoDlg::OnPlaceCallClicked() 
{
	// create and display Place Call dialog
	CXpaDemoPlaceCallDlg *dlg = new CXpaDemoPlaceCallDlg(this);
}

//
// "Clear Call" button clicked
//
void CXpaDemoDlg::OnClearCallClicked() 
{
	// create and display Clear Call dialog
	CXpaDemoClearCallDlg *dlg = new CXpaDemoClearCallDlg(this);
}

//
// "Open Data Transfer Window" button clicked
//
void CXpaDemoDlg::OnOpenDataClicked() 
{
	// pickup the lcn
	UpdateData( TRUE );

	// create and display Data Transfer dialog
	CXpaDemoDataDlg	*dlg = new CXpaDemoDataDlg(m_dataLcn, this);
}

//
//  WMXPA_ASY_INCALL handler
//
afx_msg LRESULT CXpaDemoDlg::OnInCall(WPARAM wParam, LPARAM lParam)
{
	// create and display Incoming Call dialog
	CXpaDemoInCallDlg *dlg = new CXpaDemoInCallDlg( (int)wParam, (XPA_MSG_DATA *)lParam, this );

	// we will notify API about proccing this message from within
	// CXpaDemoInCallDlg constructor because we will need the message data
	// in there, and we want to make sure it's still around ( xpaMsgComplete
	// will tell the API that it can free the message data )
	return 0;
}

//
//  WMXPA_ASY_CLEARREQ handler
//
afx_msg LRESULT CXpaDemoDlg::OnClearReq(WPARAM wParam, LPARAM lParam)
{
	// see if we need to display the Clear Request dialog box or somebody
	// else will take care of that ...
	if (gDisplayClearReq[wParam]) {
		// yes - display the Clear Request dialog box ... note that 
		// the message acknowledgement will be performed from there
		// since we will need to have message data available 
		CXpaDemoClearReqDlg *dlg = new CXpaDemoClearReqDlg( (int)wParam, (XPA_MSG_DATA *)lParam, this );
	} else {
		// nope - somebody else will take care of that, so just acknowledge the 
		// message 
		xpaMsgComplete( (XPA_MSG_DATA *)lParam );
	}
	return 0;
}

//
// Default message handler - we need it to handle correctly all unhandled 
// WMXPA_ messages ...
//
LRESULT CXpaDemoDlg::DefWindowProc( UINT message, WPARAM wParam, LPARAM lParam )
{
	if ((message >= WMXPA_ASY_INCALL) && (message <= WMXPA_RXDATA)) {
		// notify the API that we've processed the message 
		xpaMsgComplete( (XPA_MSG_DATA *)lParam );
	}
	// call the parent class 
	return CDialog::DefWindowProc( message, wParam, lParam );
}


